// Prend une matrice de capacites en entree et calcule
// le flot maximum du graphe correspondant (algo de Ford Fulkerson).

#include <iostream>

#define T 100 // nombre max de sommets
#define INIT_MIN 1000000 // capacite maximale dans le graphe

int capacites[T][T]; // 0 si pas d'arete

int residuels[T][T]; // graphe des residuels

int flot[T][T];

int n; // nombre de sommets

int s, t; // sommets source et destination

// pour le parcours en largeur
int file[T];
int debut, fin;
bool traite[T];
int pere[T]; // pour remonter le chemin trouve

void lire_entree()
{

  std::cin >> n >> s >> t;

  for (int i=0; i < n; ++i)
    for (int j=0; j < n; ++j)
      {
	std::cin >> capacites[i][j];
	residuels[i][j] = capacites[i][j];
      }
}

void initialisation()
{
  for (int i=0; i < n; ++i)
    for (int j=0; j < n; ++j)
      flot[i][j]=0;
}

bool trouve_chemin()
  // parcours en largeur du graphe des residuels
{
  debut=0; fin=0; file[0]=s;
  for (int i=0; i < n; ++i)
    traite[i]=false;
  while (debut <= fin)
    {
      for (int i=0; i<n; ++i)
	if (!traite[i] && residuels[file[debut]][i] > 0)
	  {
	    ++fin;
	    file[fin]=i;
	    traite[i] = true;
	    pere[i] = file[debut];
	    if (i==t)
	      return true;
	  }
      ++debut;
    }

  return false;
}

void affiche_flot()
{
  std::cout << "Flot max :\n";
  for (int i=0; i<n; ++i)
    {
      for (int j=0; j<n-1; ++j)
	std::cout << flot[i][j] << ' ';
      std::cout << flot[i][n-1] << std::endl;
    }
}

int min_chemin()
{
  int i = t;
  int min = INIT_MIN;
  while (pere[i] != s)
    {
      if (residuels[pere[i]][i] < min)
	min = residuels[pere[i]][i];
      i = pere[i];
    }
  if (residuels[s][i] < min)
    min = residuels[s][i];
  return min;
}

void augmente_chemin(int m)
{
  int i=t;
  while (pere[i] != s)
    {
      flot[pere[i]][i] += m;
      flot[i][pere[i]] -= m;
      residuels[pere[i]][i] = capacites[pere[i]][i]-flot[pere[i]][i];
      residuels[i][pere[i]] = capacites[i][pere[i]]-flot[i][pere[i]];
      i = pere[i];
    }
  flot[s][i] += m;
  flot[i][s] -= m;
  residuels[s][i] = capacites[s][i]-flot[s][i];;
  residuels[i][s] = capacites[i][s]-flot[i][s];;
}

int valeur_flot()
{
  int val=0;

  for (int i=0; i<n; ++i)
    val += flot[s][i];
  return val;
}

int main()
{

  lire_entree();

  initialisation();

  while(trouve_chemin())
    {
      augmente_chemin(min_chemin());
    }

  affiche_flot();

  std::cout << "Capacite flot : " << valeur_flot() << std::endl;

}
